from django.utils.translation import ugettext_lazy as _
from django.urls import reverse
from cms.utils.urlutils import admin_reverse
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool


@toolbar_pool.register
class PCartCoreToolbar(CMSToolbar):
    _pcart_menu = None

    def populate(self):
        if not self._pcart_menu:
            self._pcart_menu = self.toolbar.get_or_create_menu('pcart-core-menu', _('Core'))
            self._pcart_menu.add_sideframe_item(_('System info'), url=reverse('pcart_core:pcart-admin-system-info'))
            self._pcart_menu.add_sideframe_item(
                name=_('Theme settings'),
                url=admin_reverse('pcart_core_themesettings_changelist'),
            )
            self._pcart_menu.add_sideframe_item(
                name=_('Root files'),
                url=admin_reverse('pcart_core_rootfile_changelist'),
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 20:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_core', '0004_auto_20170222_2226'),
    ]

    operations = [
        migrations.AddField(
            model_name='rootfile',
            name='base64_decode',
            field=models.BooleanField(default=False, help_text='Use this option if you use binary data encoded with Base64 instead a plain text.', verbose_name='Base64 decode'),
        ),
    ]



from django.db import models
from django.contrib.sites.models import Site
from django.contrib.postgres.fields import JSONField, ArrayField
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
# from parler.models import TranslatableModel, TranslatedFields
import uuid
import json


# class ThemeSettings(TranslatableModel):
class ThemeSettings(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    site = models.OneToOneField(Site, verbose_name=_('Site'), on_delete=models.PROTECT, related_name='settings')

    data = JSONField(_('Data'), default=dict, blank=True)

    # translations = TranslatedFields(
    #     data=JSONField(_('Data'), default=dict, blank=True),
    # )

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Theme settings')
        verbose_name_plural = _('Theme settings')

    def __str__(self):
        return str(self.site)

    def get_upload_path(self, filename=None):
        from .utils import get_settings_upload_path
        return get_settings_upload_path(self.site_id, filename)

    def get_settings_fields(self):
        from django.core.files.storage import default_storage
        storage = default_storage
        result = json.loads(render_to_string('pcart/settings_schema.json'))

        _current = self.data.get('current', {})

        for group in result:
            for o in group.get('settings', []):
                if 'id' in o:
                    if o.get('type') in ['radio', 'select', 'text', 'collection', 'blog', 'link_list', 'color', 'textarea']:
                        o['value'] = _current.get(o['id'], o.get('default', ''))
                    elif o.get('type') in ['image']:
                        value = _current.get(o['id'], o.get('default', ''))
                        if value:
                            value = self.get_upload_path(value)
                            o['url'] = storage.url(value)
                        o['value'] = value
                    elif o.get('type') in ['checkbox']:
                        o['value'] = _current.get(o['id'], o.get('default', False))
        return result


class RootFile(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    site = models.ForeignKey(Site, verbose_name=_('Site'), on_delete=models.PROTECT, related_name='root_files')

    file_name = models.CharField(_('File name'), max_length=255)
    content = models.TextField(_('Content'), default='', blank=True)
    content_type = models.CharField(_('Content type'), default='text/plain', max_length=70)
    base64_decode = models.BooleanField(
        _('Base64 decode'), default=False,
        help_text=_('Use this option if you use binary data encoded with Base64 instead a plain text.')
    )

    cache_timeout = models.PositiveIntegerField(_('Cache timeout (seconds)'), default=3600)

    published = models.BooleanField(_('Published'), default=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Root file')
        verbose_name_plural = _('Root files')
        unique_together = ['site', 'file_name']

    def __str__(self):
        return str(self.file_name)

    def save(self, *args, **kwargs):
        self.clear_cache()
        super(RootFile, self).save(*args, **kwargs)

    def cache_key(self):
        return 'rootfile-%s-%s' % (self.site_id, self.file_name)

    def clear_cache(self):
        from django.core.cache import cache
        cache.delete(self.cache_key())

    def view(self, request):
        from django.http import HttpResponse
        from django.core.cache import cache
        response = cache.get(self.cache_key())
        if response is None:
            content = self.content
            if self.base64_decode:
                import base64
                content = base64.b64decode(content)
            response = HttpResponse(content, content_type=self.content_type)
            cache.set(self.cache_key(), response, self.cache_timeout)
        return response



import sys
import django
import platform
import getpass
import os
import psutil
import pip
# from django.utils.translation import ugettext_lazy as _
from django.core.cache import cache
from django.urls import reverse
from django.contrib import admin
from django.shortcuts import redirect, render
from .apps import PCartCoreConfig


@admin.site.admin_view
def pcart_system_info_view(request):
    PID = os.getpid()
    context = {
        'opts': {
            'app_label': PCartCoreConfig.name,
            'app_config': {
                'verbose_name': PCartCoreConfig.verbose_name,
            },
        },
        'python_version': sys.version,
        'django_version': django.get_version(),
        'os_version': platform.platform(),
        'virtualenv_dir': getattr(sys, 'prefix', 'sys.prefix is not defined.'),
        'system_user': getpass.getuser(),
        'memory_stats': psutil.virtual_memory(),
        'pid': PID,
        'current_process': psutil.Process(PID),
        'libraries': sorted(pip.get_installed_distributions(local_only=True), key=lambda x: x.key),
    }
    return render(request, 'admin/pcart_core/system_info.html', context)


@admin.site.admin_view
def pcart_clear_cache_view(request):
    cache.clear()
    return redirect(reverse('pcart_core:pcart-admin-system-info'))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 18:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('pcart_core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RootFile',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('file_name', models.CharField(max_length=255, verbose_name='File name')),
                ('content', models.TextField(blank=True, default='', verbose_name='Content')),
                ('cache_timeout', models.PositiveIntegerField(default=3600, verbose_name='Cache timeout (seconds)')),
                ('published', models.BooleanField(default=True, verbose_name='Published')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='root_files', to='sites.Site', verbose_name='Site')),
            ],
            options={
                'verbose_name_plural': 'Root files',
                'verbose_name': 'Root file',
            },
        ),
    ]



from django.contrib.sites.shortcuts import get_current_site
from django.middleware.common import CommonMiddleware
from django import http
from .models import RootFile


class PCartCommonMiddleware(CommonMiddleware):
    response_redirect_class = http.HttpResponseRedirect


class RootFileRouterMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def _get_root_file_instance(self, site, path):
        from django.core.cache import cache
        cache_key = 'rootfiles-list-%s' % site.id
        root_files_route = cache.get(cache_key)
        if root_files_route is None:
            root_files_route = {
                '/%s' % k: v
                for k, v in RootFile.objects.filter(site=site, published=True).values_list('file_name', 'id')}
            cache.set(cache_key, root_files_route, 3600)
        _id = root_files_route.get(path)
        if _id is not None:
            try:
                result = RootFile.objects.get(pk=_id)
                return result
            except RootFile.DoesNotExist:
                pass

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        site = get_current_site(request)
        root_file = self._get_root_file_instance(site, request.get_full_path())
        if root_file is not None:
            response = root_file.view(request)
            return response

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response



from django.conf.urls import url
from . import views


urlpatterns = [
    url(
        r'^system-info/$',
        views.pcart_system_info_view,
        name='pcart-admin-system-info'),
    url(
        r'^clear-cache/$',
        views.pcart_clear_cache_view,
        name='pcart-admin-clear-cache'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 20:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('pcart_core', '0003_rootfile_content_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rootfile',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='root_files', to='sites.Site', verbose_name='Site'),
        ),
        migrations.AlterUniqueTogether(
            name='rootfile',
            unique_together=set([('site', 'file_name')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 18:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_core', '0002_rootfile'),
    ]

    operations = [
        migrations.AddField(
            model_name='rootfile',
            name='content_type',
            field=models.CharField(default='text/plain', max_length=70, verbose_name='Content type'),
        ),
    ]



from django import template
from django.utils.safestring import mark_safe
from django.conf import settings

register = template.Library()

# Debugging

@register.simple_tag
def debug_print(*values):
    if settings.DEBUG:
        print(*values)
        return values
    else:
        return ''


@register.simple_tag(takes_context=True)
def debug_context(context):
    if settings.DEBUG:
        print(context)
        return context
    else:
        return ''


# Helpful tags and filters

@register.simple_tag
def random():
    import random
    return random.random()


@register.simple_tag
def randint(a, b):
    import random
    return random.randint(a, b)


@register.simple_tag
def timezone_now():
    from django.utils import timezone
    return timezone.now()


@register.filter
def get_element(iterable, index):
    return iterable.get(index)


@register.filter
def index_element(iterable, index):
    return iterable[index]


@register.filter
def to_list(obj):
    return list(obj)


@register.filter
def to_set(obj):
    return set(obj)


@register.filter
def to_dict(obj):
    return dict(obj)


@register.filter
def to_int(obj):
    return int(obj)


@register.filter
def to_float(obj):
    return float(obj)


@register.filter
def to_decimal(obj):
    from decimal import Decimal
    return Decimal(obj)


@register.filter
def ignore_none(value):
    return value or ''


@register.filter
def sort(iterable):
    return iterable.sort()


@register.filter
def sort_by_attr(iterable, attr):
    result = sorted(iterable, key=lambda x: getattr(x, attr))
    return result


@register.filter
def sort_by_key(iterable, key):
    result = sorted(iterable, key=lambda x: x[key])
    return result


@register.filter
def widget_css_class(field, css_class):
    return field.as_widget(attrs={"class": css_class})


@register.filter
def widget_placeholder(field, value):
    return field.as_widget(attrs={"placeholder": value})


@register.simple_tag
def widget_attrs(field, **kwargs):
    return field.as_widget(attrs=kwargs)


@register.filter
def markdown(text):
    import markdown
    return mark_safe(markdown.markdown(text))


@register.filter
def subtract(value, arg):
    return value - arg


@register.filter
def add(value, arg):
    return value + arg


@register.filter
def multiply(value, arg):
    return value * arg


@register.filter
def divide(value, arg):
    return value / arg


@register.filter
def append_to_list(value, arg):
    return value.append(arg)


@register.simple_tag(takes_context=True)
def theme_url(context, filename):
    from ..utils import get_settings_upload_url
    return get_settings_upload_url(context['site'].id, filename)


@register.simple_tag
def optional_url(url_name, *args, **kwargs):
    from django.urls import reverse, NoReverseMatch
    try:
        return reverse(url_name, args=args, kwargs=kwargs)
    except NoReverseMatch:
        return '#unknown-url'


@register.filter
def is_checkbox(field):
    from django.forms import CheckboxInput
    return isinstance(field.field.widget, CheckboxInput)


@register.filter
def is_hidden_input(field):
    from django.forms import HiddenInput
    return isinstance(field.field.widget, HiddenInput)


@register.filter
def weight(value):
    return '%.1f kg' % value


@register.simple_tag(takes_context=True)
def render_string(context, template_string, **kwargs):
    """
    Renders `template_string` content as a Django template with given context.
    """
    from django.template import Context, Template
    _header = '{% load i18n l10n pcart_core staticfiles %}'
    _template = _header + template_string

    t = Template(_template)
    _context = context
    _context.update(kwargs)
    c = Context(_context)
    return mark_safe(t.render(c))


@register.simple_tag
def create_list(*args):
    return args


@register.simple_tag
def create_dict(**kwargs):
    return kwargs


@register.simple_tag
def save(value):
    return value


@register.simple_tag
def create_list():
    return list()


@register.simple_tag
def create_dict():
    return dict()


@register.filter
def attrs_list(obj):
    return dir(obj)


@register.filter
def inspect(obj):
    attrs = dir(obj)
    result = dict()
    for attr in attrs:
        result[attr] = getattr(obj, attr)
    return result


@register.simple_tag
def split_str(value, delimiter=','):
    return value.split(delimiter)


@register.filter
def money(value, template_name='price_template'):
    if value is None or value == '':
        value = 0.0

    _template = '<span class="money" data-value="%s" data-currency="%s">%s</span>'
    currency_code = getattr(settings, 'PCART_DEFAULT_CURRENCY', 'default')

    if hasattr(settings, 'PCART_CURRENCIES'):
        currency_config = settings.PCART_CURRENCIES.get(currency_code)
        return mark_safe(
            _template % (
                value,
                currency_code,
                currency_config.get(template_name, '%s') % value)
            )
    else:
        return _template % (value, currency_code, value)


@register.simple_tag
def set_query_parameter(url, param_name, param_value):
    from urllib.parse import (
        urlencode,
        parse_qs,
        urlsplit,
        urlunsplit,
    )
    scheme, netloc, path, query_string, fragment = urlsplit(url)
    query_params = parse_qs(query_string)
    query_params[param_name] = [param_value]
    new_query_string = urlencode(query_params, doseq=True)
    return urlunsplit((scheme, netloc, path, new_query_string, fragment))


@register.simple_tag(takes_context=True)
def build_absolute_uri(context, location):
    return context['request'].build_absolute_uri(location)


@register.filter(name='len')
def _len(sequence):
    return len(sequence)


# Querysets

@register.filter
def order_by(queryset, values):
    values_attrs = values.split(',')
    return queryset.order_by(*values_attrs)


@register.filter
def select_related(queryset, value):
    return queryset.select_related(value)


@register.filter
def prefetch_related(queryset, value):
    return queryset.prefetch_related(value)


@register.filter
def distinct(queryset):
    return queryset.distinct()


@register.simple_tag
def prepare_query_object(logic='AND', **kwargs):
    from django.db.models import Q
    query = None
    for k,v in kwargs.items():
        if query is None:
            query = Q(**{k: v})
        else:
            if logic == 'AND':
                query &= Q(**{k: v})
            elif logic == 'OR':
                query |= Q(**{k: v})
    return query


@register.filter
def filter_queryset(queryset, query):
    return queryset.filter(query)


@register.filter
def exclude_from_queryset(queryset, query):
    return queryset.exclude(query)

# Liquid compatibility


@register.filter
def append(value, arg):
    """Appends characters to a string.
    Input:
        {{ 'sales' | append: '.jpg' }}
    Output:
        sales.jpg
    """
    return value + arg


@register.filter
def camelcase(value):
    """Converts a string into CamelCase.
    Input:
        {{ 'coming-soon' | camelcase }}
    Output:
        ComingSoon
    """
    chunks = value.split('-')
    return ''.join([x.title() for x in chunks])


@register.filter
def capitalize(value):
    """Capitalizes the first word in a string.
    Input:
        {{ 'capitalize me' | capitalize }}
    Output:
        Capitalize me
    """
    return value.title()


@register.filter
def downcase(value):
    """Converts a string into lowercase.
    Input
        {{ 'UPPERCASE' | downcase }}
    Output
        uppercase
    """
    return value.lower()


@register.filter
def escape(value):
    """Escapes a string.
    Input
        {{ "<p>test</p>" | escape }}
    Output
         <!-- The <p> tags are not rendered -->
        <p>test</p>"""
    import html
    return html.escape(value)


@register.filter
def upcase(value):
    return value.upper()


# capture tag

@register.tag(name='capture')
def do_capture(parser, token):
    """
    Capture the contents of a tag output.
    Usage:
    .. code-block:: html+django
        {% capture %}..{% endcapture %}                    # output in {{ capture }}
        {% capture silent %}..{% endcapture %}             # output in {{ capture }} only
        {% capture as varname %}..{% endcapture %}         # output in {{ varname }}
        {% capture as varname silent %}..{% endcapture %}  # output in {{ varname }} only
    For example:
    .. code-block:: html+django
        {# Allow templates to override the page title/description #}
        <meta name="description" content="{% capture as meta_description %}{% block meta-description %}{% endblock %}{% endcapture %}" />
        <title>{% capture as meta_title %}{% block meta-title %}Untitled{% endblock %}{% endcapture %}</title>
        {# copy the values to the Social Media meta tags #}
        <meta property="og:description" content="{% block og-description %}{{ meta_description }}{% endblock %}" />
        <meta name="twitter:title" content="{% block twitter-title %}{{ meta_title }}{% endblock %}" />
    """
    bits = token.split_contents()

    # tokens
    t_as = 'as'
    t_silent = 'silent'
    var = 'capture'
    silent = False

    num_bits = len(bits)
    if len(bits) > 4:
        raise template.TemplateSyntaxError("'capture' node supports '[as variable] [silent]' parameters.")
    elif num_bits == 4:
        t_name, t_as, var, t_silent = bits
        silent = True
    elif num_bits == 3:
        t_name, t_as, var = bits
    elif num_bits == 2:
        t_name, t_silent = bits
        silent = True
    else:
        var = 'capture'
        silent = False

    if t_silent != 'silent' or t_as != 'as':
        raise template.TemplateSyntaxError("'capture' node expects 'as variable' or 'silent' syntax.")

    nodelist = parser.parse(('endcapture',))
    parser.delete_first_token()
    return CaptureNode(nodelist, var, silent)


class CaptureNode(template.Node):
    def __init__(self, nodelist, varname, silent):
        self.nodelist = nodelist
        self.varname = varname
        self.silent = silent

    def render(self, context):
        output = self.nodelist.render(context)
        context[self.varname] = output
        if self.silent:
            return ''
        else:
            return output




SLUG_SAVE_AS_DASH = '.,/'
SLUG_ALLOWED_CHARS = r'^[a-zA-Z0-9\-]$'


def get_settings_upload_path(site_id, filename):
    import os
    path = os.path.join(
        'theme-config',
        'site-%s' % site_id,
    )
    if filename is not None:
        path = os.path.join(path, filename)
    return path


def get_settings_upload_url(site_id, filename):
    from django.core.files.storage import default_storage
    path = get_settings_upload_path(site_id, filename)
    return default_storage.url(path)


def slugify_unicode(value, save_as_dash=SLUG_SAVE_AS_DASH, dash='-'):
    import re
    import slugify
    for k in save_as_dash:
        value = value.replace(k, dash)
    result = slugify.slugify(value, only_ascii=True)

    pattern = re.compile(SLUG_ALLOWED_CHARS)
    result = ''.join(filter(lambda x: pattern.match(x) is not None, result))
    return result


def get_unique_slug(value, model_class, slug_attr='slug', slug_func=slugify_unicode, ignore_slugs=[]):
    """ Generate unique slug for a model."""
    import itertools
    _slug = _orig = slug_func(value)
    for x in itertools.count(1):
        if not model_class.objects.exclude(
                **{'%s__in' % slug_attr: ignore_slugs}).filter(**{slug_attr: _slug}).exists():
            break
        _slug = '%s-%d' % (_orig, x)
    return _slug


def markdown_to_html(source):
    import markdown
    html = markdown.markdown(
        source,
        extensions=[
            'markdown.extensions.nl2br',
            'urlize',
        ])
    return html
